/***********************************************************************************/
/**	\file				Ingame.cpp
**	\brief				Implementation zur Klasse Ingame
*************************************************************************************
**	Autor:				Christian Roesch
*************************************************************************************
**	-tut nichts-
**
*//*********************************************************************************/


// includes
#include "Ingame.h"

#include "../framework.h"
#include "../ScreenModes.h"

#include <vector>
#include <ctime>
#include <sstream>

#include <fstream>


// Methoden-Definitionen

Ingame::Ingame( FrameWork::Config* pSettings )
:
m_pFont1( NULL ),
m_Board( m_iBoardLengthX, m_iBoardLengthY ),
m_bSelected( false ),
m_ptSelection( 0, 0 ),
m_pSettings( pSettings )
{
	m_Board.m_pIngame= this;
}

Ingame::~Ingame()
{
	delete m_pFont1;
	m_pFont1= NULL;
}

std::string Ingame::getName() const
{
	return "Ingame";
}

void Ingame::onEnter( void* p_context )
{
	m_iRules= (int)m_pSettings->getFloat( "RULES" );
	FrameWork::ScreenModes::Mode* pMode= FrameWork::ScreenModes::inst().getActMode();

	m_fScreenX= (float)pMode->m_iWidth;
	m_fScreenY= (float)pMode->m_iHeight;
	
	m_fArrowX= pMode->getScaledFloatX( "ARROWX" );
	m_fArrowY= pMode->getScaledFloatY( "ARROWY" );

	m_fGameBoardX= pMode->getScaledFloatX( "GAMEBOARDX" );
	m_fGameBoardY= pMode->getScaledFloatY( "GAMEBOARDY" );

	m_fGameBoardStepX= pMode->getScaledFloatX( "GAMEBOARDSTEPX" );
	m_fGameBoardStepY= pMode->getScaledFloatY( "GAMEBOARDSTEPY" );

	m_fGameLevelX= pMode->getScaledFloatX( "GAMELEVELX" );
	m_fGameLevelY= pMode->getScaledFloatY( "GAMELEVELY" );

	m_fGameScoresX= pMode->getScaledFloatX( "GAMESCORESX" );
	m_fGameScoresY= pMode->getScaledFloatY( "GAMESCORESY" );

	m_fGameOverX= pMode->getScaledFloatX( "GAMEOVERX" );
	m_fGameOverY= pMode->getScaledFloatY( "GAMEOVERY" );

	m_Mouse.setClipRect( 0, m_fScreenX * 0.98f, 0, m_fScreenY * 0.98f );
	m_Mouse.setPos( FrameWork::vec3( m_fScreenX * 0.5f, m_fScreenY * 0.5f, 0 ) );

	m_pFont1=	new FrameWork::Font(
						pMode->m_cfgVirtualData.getString( "GAMEFONT1" ),
						(int)pMode->getScaledFloatX( "GAMEFONT1_X" ), 
						(int)pMode->getScaledFloatY( "GAMEFONT1_Y" ),
						0 );

	m_csArrow.load( "gfx/mouse.txt" );
	m_csBoard.load( "gfx/board.txt" );
	m_csBalls.load( "gfx/balls.txt" );
	m_csGameOver.load( "gfx/gameover.txt" );

	srand( (unsigned)time( NULL ) );

	m_iLevel= 0;
	m_iDeletedBalls= 0;
	m_iScore= 0;
	m_iInRow= 0;
	m_bGameOver= false;

	m_Board.clear();
	/*m_Board.addBalls( getBallCount(), getColorCount() );
	m_Board.appear();
	m_Board.addBalls( getBallCount(), getColorCount() );*/
	m_Board.checkPlayability();

	m_bSelected= false;

	m_listTexteffects.clear();
	m_listBalleffects.clear();
	m_listBallMoves.clear();
}

void Ingame::onLeave( void* p_context, bool quit )
{
	delete m_pFont1;
	m_pFont1= NULL;

	m_csArrow.free();
	m_csBoard.free();
	m_csBalls.free();
	m_csGameOver.free();
}

void Ingame::render( float dt )
{
	FrameWork::Graphics& graphics = FrameWork::Graphics::inst();
	FrameWork::Surface* pBackBuffer= graphics.getBackBuffer();
	pBackBuffer->clear(FrameWork::color(0.2f, 0.4f, 0.8f, 1.0f));

	int iHX;
	int iHY;
	bool bMouseOverBoard= getBoardMouseHit(iHX, iHY );
	static float fSineTime= 0;
	fSineTime+= dt;
	if( fSineTime > 628.0f )
	{
		fSineTime-= 628.0f;
	}

	for( int iY= 0; iY < m_iBoardLengthY; ++iY )
	{
		for( int iX= 0; iX < m_iBoardLengthX; ++iX )
		{
			FrameWork::vec2 vec2Pos(	m_fGameBoardX + iX * m_fGameBoardStepX,
										m_fGameBoardY + iY * m_fGameBoardStepY );

			BoardField bfAct= m_Board.get( iX, iY );

			int iFrame= 0;

			if( m_bSelected && bfAct.m_iPathStep != 0 )
			{
				iFrame= 1;
			}

			m_csBoard.drawFrame(	pBackBuffer, 
									vec2Pos,
									FrameWork::vec2( m_fGameBoardStepX, m_fGameBoardStepY ),
									iFrame );

		}
	}

	for( int iY= 0; iY < m_iBoardLengthY; ++iY )
	{
		for( int iX= 0; iX < m_iBoardLengthX; ++iX )
		{
			FrameWork::vec2 vec2Pos(	m_fGameBoardX + iX * m_fGameBoardStepX,
										m_fGameBoardY + iY * m_fGameBoardStepY );

			BoardField bfAct= m_Board.get( iX, iY );

			int iFrame= bfAct.m_iColor * 2;
			iFrame+= ( bfAct.m_iAppeared >= 0 )? 0 : 1;

			float fSize= 1.0f;
			if( ( bfAct.m_iAppeared >= 0 ) && iX == iHX && iY == iHY && !m_bSelected )
			{
				fSize= 0.98f + 0.05f * (float)sinf( fSineTime * 6.0f );
			}
			if( bfAct.m_iAppeared < 0 )
			{
				fSize= -1.0f / (float)bfAct.m_iAppeared;
			}

			if( bfAct.m_iColor >= 0 )
			{
				m_csBalls.drawFrame(	pBackBuffer, 
										vec2Pos,
										FrameWork::vec2(	m_fGameBoardStepX * fSize,
															m_fGameBoardStepY * fSize ),
										iFrame );
			}
		}
	}

	for(	std::list< BallEffect >::iterator it= m_listBalleffects.begin();
			it!= m_listBalleffects.end(); )
	{
		BallEffect& beAct= *it;
		beAct.m_fTime-= dt;

		BoardField& bfAct= m_Board.get( beAct.m_ptPos.m_iX, beAct.m_ptPos.m_iY );

		if( beAct.m_fTime < 0.0f )
		{
			it= m_listBalleffects.erase( it );
		}
		else
		{
			it++;
			switch( beAct.m_Mode )
			{
			case BallEffect::BE_EXPLODE:
				{
					float fSize= 1.0f;
					int iAlpha= 255;
					if( beAct.m_fTime < 1.0f )
					{
						iAlpha= (int)( 255.0f * beAct.m_fTime );
						fSize= ( 1.5f - beAct.m_fTime * 0.5f );
					}
					FrameWork::vec2 vec2Pos(	m_fGameBoardX + beAct.m_ptPos.m_iX * m_fGameBoardStepX,
						m_fGameBoardY + beAct.m_ptPos.m_iY * m_fGameBoardStepY );

					if( beAct.m_fTime > 1.0f && beAct.m_fTime < 1.25f)
					{
						vec2Pos.x+= m_fGameBoardStepX * FrameWork::floatRand( -0.025f, 0.025f );
						vec2Pos.y-= m_fGameBoardStepY * FrameWork::floatRand( -0.025f, 0.025f );
					}
					m_csBalls.drawFrame(	pBackBuffer,
						vec2Pos,
						FrameWork::vec2( m_fGameBoardStepX * fSize, m_fGameBoardStepY * fSize ),
						beAct.m_iColor * 2,
						0,
						iAlpha << 24 | 0xffffff );
				}
				break;
			case BallEffect::BE_APPEAR:
				{
					float fSize= 1.0f;
					int iFrame= beAct.m_iColor * 2;
					if( bfAct.m_iAppeared < 0 )
					{
						fSize= -1.0f / (float)bfAct.m_iAppeared;
						iFrame++;
					}
					int iAlpha= 255;
					if( beAct.m_fTime < 1.0f )
					{
						fSize= ( 1.0f - beAct.m_fTime * 0.5f );
					}
					FrameWork::vec2 vec2Pos(	m_fGameBoardX + beAct.m_ptPos.m_iX * m_fGameBoardStepX,
												m_fGameBoardY + beAct.m_ptPos.m_iY * m_fGameBoardStepY );

					m_csBoard.drawFrame(	pBackBuffer, 
											vec2Pos,
											FrameWork::vec2( m_fGameBoardStepX, m_fGameBoardStepY ),
											0 );
			
					m_csBalls.drawFrame(	pBackBuffer,
											vec2Pos,
											FrameWork::vec2( m_fGameBoardStepX * fSize, m_fGameBoardStepY * fSize ),
											iFrame,
											0,
											iAlpha << 24 | 0xffffff );
				}
				break;
			}
		}
	}

	for(	std::list< BallMove >::iterator it= m_listBallMoves.begin();
			it!= m_listBallMoves.end(); )
	{
		BallMove& bmAct= *it;
		bmAct.m_fTime+= dt;

		int iLength= (int)bmAct.m_vecptPos.size();
		float fMove= bmAct.m_fTime * (float)m_iMoveSpeed;
		int iFirst= (int)fMove;
		int iSecond= iFirst + 1;

		if( iSecond >= iLength )
		{
			it= m_listBallMoves.erase( it );
		}
		else
		{
			it++;
			float fWeight= fMove - (float)iFirst;
			float fAntiWeight= 1.0f - fWeight;

			Point ptLast= bmAct.m_vecptPos[ iLength - 1 ];

			FrameWork::vec2 vec2Pos(	m_fGameBoardX + ptLast.m_iX * m_fGameBoardStepX,
										m_fGameBoardY + ptLast.m_iY * m_fGameBoardStepY );

			m_csBoard.drawFrame(	pBackBuffer, 
									vec2Pos,
									FrameWork::vec2( m_fGameBoardStepX, m_fGameBoardStepY ),
									0 );
			
			Point ptFirst= bmAct.m_vecptPos[ iFirst ];
			Point ptSecond= bmAct.m_vecptPos[ iSecond ];

			FrameWork::vec2 vec2First(	m_fGameBoardX + ptFirst.m_iX * m_fGameBoardStepX,
										m_fGameBoardY + ptFirst.m_iY * m_fGameBoardStepY );
			FrameWork::vec2 vec2Second(	m_fGameBoardX + ptSecond.m_iX * m_fGameBoardStepX,
										m_fGameBoardY + ptSecond.m_iY * m_fGameBoardStepY );

			m_csBalls.drawFrame(	pBackBuffer,
									vec2First * fAntiWeight + vec2Second * fWeight,
									FrameWork::vec2( m_fGameBoardStepX, m_fGameBoardStepY ),
									bmAct.m_iColor * 2 );
		}
	}

	std::ostringstream ossLevel;
	std::ostringstream ossScore;

	ossLevel << "Level\n" << ( m_iLevel + 1 );
	ossScore << "Score\n" << m_iScore;

	m_pFont1->drawText( 
		pBackBuffer,
		FrameWork::vec2( m_fGameLevelX, m_fGameLevelY ),
		ossLevel.str(),
		FrameWork::DTF_HCENTER|FrameWork::DTF_VCENTER,
		0xff000000 );
	m_pFont1->drawText( 
		pBackBuffer,
		FrameWork::vec2( m_fGameScoresX, m_fGameScoresY ),
		ossScore.str(),
		FrameWork::DTF_HCENTER|FrameWork::DTF_VCENTER,
		0xff000000 );

	for(	std::list< TextEffect >::iterator it= m_listTexteffects.begin();
			it!= m_listTexteffects.end(); )
	{
		TextEffect& teAct= *it;
		teAct.m_fTime+= dt;

		if( teAct.m_fTime >= 5.0f )
		{
			it= m_listTexteffects.erase( it );
		}
		else
		{
			int iAlpha= 255;
			if( teAct.m_fTime >= 1.0f )
			{
				iAlpha -= (int)( 63.5f * ( teAct.m_fTime - 1.0f ) );
			}
				
			m_pFont1->drawText( 
				pBackBuffer,
				teAct.m_vec2Pos,
				teAct.m_strText,
				FrameWork::DTF_HCENTER|FrameWork::DTF_VCENTER,
				iAlpha << 24 );

			it++;
		}
	}

	if( m_bGameOver)
	{
		m_csGameOver.drawFrame(
			pBackBuffer,
			FrameWork::vec2( m_fScreenX * 0.5f, m_fScreenY * 0.5f ),
			FrameWork::vec2( m_fGameOverX, m_fGameOverY ),
			0 );
	}


	FrameWork::vec3 vec3Pos= m_Mouse.getPos();
	m_csArrow.drawFrame(	pBackBuffer,
							FrameWork::vec2( vec3Pos.x + m_fArrowX / 2, vec3Pos.y + m_fArrowY / 2 ),
							FrameWork::vec2( m_fArrowX, m_fArrowY ),
							0 );

	graphics.showBackBuffer( 1.0f );
}

void Ingame::update( float dt )
{
	m_Mouse.update();
	FrameWork::Input& input = FrameWork::Input::inst();

	if( input.wasKeyboardKeyPressed( DIK_ESCAPE ) )
	{
		getGame()->setGameState( "Menu" );
	}

	if( m_bGameOver )
	{
		return;
	}

	int iX;
	int iY;
	bool bMouseOverBoard= getBoardMouseHit(iX, iY );

	if( m_bSelected && m_Mouse.isMouseMove() )
	{
		if( bMouseOverBoard )
		{
			if( !m_Board.findPath( m_ptSelection, Point( iX, iY ) ) )
			{
				m_Board.clearPath();
			}
		}
		else
		{
			m_Board.clearPath();
		}
	}

	if( m_Mouse.isButtonClick( 0 ) )
	{
		if( m_bSelected )
		{
			//Kugel ziehen
			if( bMouseOverBoard )
			{
				if( iX != m_ptSelection.m_iX || iY != m_ptSelection.m_iY )
				{
					if( m_Board.isFree( iX, iY ) &&
						m_Board.findPath( m_ptSelection, Point( iX, iY ) ) )
					{
						m_bSelected= false;

						int iDeleted= 
							m_Board.move( m_ptSelection, Point( iX, iY ) );
						if( iDeleted == 0 )
						{
							iDeleted= m_Board.appear();
							int iAdded= m_Board.addBalls( 
								getBallCount(), 
								getColorCount() );
							if( iDeleted == 0 )
							{
								m_iInRow= 0;
							}
						}
						else
						{
							score( iDeleted, Point( iX, iY ) );
						}

						if( !m_Board.checkPlayability() )
						{
							m_bGameOver= true;
						}
					}
				}
			}
			else
			{
				m_bSelected= false;
			}
		}
		else
		{
			if( bMouseOverBoard && !m_Board.isFree( iX, iY ) )
			{
				m_bSelected= true;
				m_ptSelection= Point( iX, iY );
				if( !m_Board.findPath( m_ptSelection, Point( iX, iY ) ) )
				{
					m_Board.clearPath();
				}
			}
		}
	}

	if( m_Mouse.isButtonClick( 1 ) )
	{
		m_bSelected= false;
	}

}

Ingame::Board::Board( int iX, int iY )
{
	m_vecbfData.resize( iX * iY );
	m_iSizeX= iX;
	m_iSizeY= iY;
	clear();
}

void Ingame::Board::clear()
{
	for( int iY= 0; iY < m_iSizeY; ++iY )
	{
		for( int iX= 0; iX < m_iSizeX; ++iX )
		{
			set( iX, iY, BoardField() );
		}
	}
}

Ingame::BoardField& Ingame::Board::get( int iX, int iY )
{
	static BoardField bfDummy( Ingame::m_iMaxColors, true );
	if( iX < 0 ||
		iX >= m_iSizeX ||
		iY < 0 ||
		iY >= m_iSizeY )
	{
		return bfDummy;
	}

	return m_vecbfData[ iX + iY * m_iSizeX ];
}

void Ingame::Board::set( int iX, int iY, BoardField bfNew )
{
	if( iX < 0 ||
		iX >= m_iSizeX ||
		iY < 0 ||
		iY >= m_iSizeY )
	{
		return;
	}

	m_vecbfData[ iX + iY * m_iSizeX ]= bfNew;
}

bool Ingame::Board::isFree( int iX, int iY )
{
	if( iX < 0 ||
		iX >= m_iSizeX ||
		iY < 0 ||
		iY >= m_iSizeY )
	{
		return false;
	}

	return ( get( iX, iY).m_iColor == -1 || !( get( iX, iY).m_iAppeared >= 0 ) );
}

int Ingame::Board::appear()
{
	int iDeleted= 0;

	static std::vector< Point > vecptNew;
	vecptNew.clear();

	for( int iY= 0; iY < m_iSizeY; ++iY )
	{
		for( int iX= 0; iX < m_iSizeX; ++iX )
		{
			BoardField bfAct= get( iX, iY );

			if( bfAct.m_iColor != -1 )
			{
				if( bfAct.m_iAppeared < 0 ) 
				{
					bfAct.m_iAppeared++;
					set( iX, iY, bfAct );
					if( bfAct.m_iAppeared >= 0 )
					{
						vecptNew.push_back( Point( iX, iY ) );
					}

					BallEffect beNew;
					beNew.m_fTime= 1.0f;
					beNew.m_iColor= bfAct.m_iColor;
					beNew.m_Mode= BallEffect::BE_APPEAR;
					beNew.m_ptPos= Point( iX, iY );

					m_pIngame->m_listBalleffects.push_back( beNew );
				}
			}
		}
	}

	for( int i= 0; i < (int)vecptNew.size(); ++i )
	{
		int iRet= chain( vecptNew[ i ], m_iMinChain, 1.0f );
		if( iRet > 0 )
		{
			iDeleted+= iRet;
			m_pIngame->score( iRet, vecptNew[ i ] );
		}
	}

	return iDeleted;
}

int Ingame::Board::addBalls( int iCount, int iColors )
{
	static std::vector< Point > vecFree;
	vecFree.clear();
	for( int iY= 0; iY < m_iSizeY; ++iY )
	{
		for( int iX= 0; iX < m_iSizeX; ++iX )
		{
			if( get( iX, iY ).m_iColor == -1 )
			{
				vecFree.push_back( Point( iX, iY ) ) ;
			}
		}
	}

	if( iCount > (int)vecFree.size() )
	{
		iCount= (int)vecFree.size();
	}

	int iRet= iCount;

	while( iCount > 0 )
	{
		int iFree= (int)vecFree.size();
		int iRand= rand() % iFree;
		int iX= vecFree[ iRand ].m_iX;
		int iY= vecFree[ iRand ].m_iY;
		vecFree[ iRand ]= vecFree[ iFree - 1 ];
		vecFree.pop_back();

		BoardField bfNew(	rand() % iColors,
							m_pIngame->m_iRules == 0 ? -2 : -1);
		set( iX, iY, bfNew );
		iCount--;
	}

	return iRet;
}

bool Ingame::Board::findPath( Point ptStart, Point ptEnd )
{
	clearPath();

	get( ptStart.m_iX, ptStart.m_iY ).m_iPathStep= 1;

	std::list< Point > listNodes;
	
	listNodes.push_back( ptStart );
	int iLength= 1;

	while( true )
	{
		iLength++;
		std::list< Point > listNewNodes;

		std::list< Point >::iterator it;
		for(	it= listNodes.begin(); 
				it !=  listNodes.end();
				++it )
		{
			Point ptAct= *it;

			ptAct.m_iX++;
			if( isFree( ptAct.m_iX, ptAct.m_iY ) && get( ptAct.m_iX, ptAct.m_iY ).m_iPathStep == 0 )
			{
				get( ptAct.m_iX, ptAct.m_iY ).m_iPathStep= iLength;
				listNewNodes.push_back( ptAct );
			}
			ptAct.m_iX-= 2;
			if( isFree( ptAct.m_iX, ptAct.m_iY ) && get( ptAct.m_iX, ptAct.m_iY ).m_iPathStep == 0 )
			{
				get( ptAct.m_iX, ptAct.m_iY ).m_iPathStep= iLength;
				listNewNodes.push_back( ptAct );
			}
			ptAct.m_iX++;
			ptAct.m_iY++;
			if( isFree( ptAct.m_iX, ptAct.m_iY ) && get( ptAct.m_iX, ptAct.m_iY ).m_iPathStep == 0 )
			{
				get( ptAct.m_iX, ptAct.m_iY ).m_iPathStep= iLength;
				listNewNodes.push_back( ptAct );
			}
			ptAct.m_iY-= 2;
			if( isFree( ptAct.m_iX, ptAct.m_iY ) && get( ptAct.m_iX, ptAct.m_iY ).m_iPathStep == 0 )
			{
				get( ptAct.m_iX, ptAct.m_iY ).m_iPathStep= iLength;
				listNewNodes.push_back( ptAct );
			}
		}

		if( get( ptEnd.m_iX, ptEnd.m_iY ).m_iPathStep != 0 )
		{
			break;
		}

		if( listNewNodes.size() == 0 )
		{
			return false;
		}

		listNodes.clear();
		listNodes= listNewNodes;
	}

	int iDirOffset= 0;

	int iDX= ptEnd.m_iX - ptStart.m_iX;
	int iDY= ptEnd.m_iY - ptStart.m_iY;

	if( abs( iDX ) > abs( iDY ) )
	{
		iDirOffset= iDX > 0 ? 2 : 0;
	}
	else
	{
		iDirOffset= iDY > 0 ? 3 : 1;
	}


	while( ptEnd.m_iX != ptStart.m_iX || ptEnd.m_iY != ptStart.m_iY )
	{
		int iAct= get( ptEnd.m_iX, ptEnd.m_iY ).m_iPathStep;
		get( ptEnd.m_iX, ptEnd.m_iY ).m_iPathStep= -iAct;
		int i;
		for( i= 0; i < 4; ++i )
		{
			int iDir= iDirOffset + i;

			int iX= ptEnd.m_iX;
			int iY= ptEnd.m_iY;
			switch( iDir % 4  )
			{
			case 0:
				iX++;
				break;
			case 1:
				iY++;
				break;
			case 2:
				iX--;
				break;
			case 3:
				iY--;
				break;
			}
			if( get( iX, iY ).m_iPathStep == iAct - 1 )
			{
				ptEnd.m_iX= iX;
				ptEnd.m_iY= iY;
				iDirOffset= iDir;
				break;
			}
		}
		assert( i < 5 );
		if( i == 5 )
		{
			return false;
		}
	}

	get( ptEnd.m_iX, ptEnd.m_iY ).m_iPathStep= -1;

	for( int iY= 0; iY < m_iSizeY; ++iY )
	{
		for( int iX= 0; iX < m_iSizeX; ++iX )
		{
			if( get( iX, iY ).m_iPathStep > 0 )
			{
				get( iX, iY ).m_iPathStep= 0;
			}
		}
	}

	/*std::ofstream ofs( "error.log", std::ios::out );

	for( int iY= 0; iY < m_iSizeY; ++iY )
	{
		for( int iX= 0; iX < m_iSizeX; ++iX )
		{
			ofs << get( iX, iY ).m_iPathStep << "\t";
		}
		ofs << "\n";
	}*/

	return true;
}

void Ingame::Board::clearPath()
{
	for( int i= 0; i < m_iSizeX * m_iSizeY; ++i )
	{
		m_vecbfData[ i ].m_iPathStep= 0;
	}
}

int Ingame::Board::move( Point ptStart, Point ptEnd )
{
	/*std::ofstream ofs( "error.log", std::ios::out|std::ios::app );

	for( int iY= 0; iY < m_iSizeY; ++iY )
	{
		for( int iX= 0; iX < m_iSizeX; ++iX )
		{
			ofs << get( iX, iY ).m_iPathStep << "\t";
		}
		ofs << "\n";
	}*/

	BoardField bfCopy= get( ptStart.m_iX, ptStart.m_iY );

	BallMove bmNew;
	for( int iY= 0; iY < m_iSizeY; ++iY )
	{
		for( int iX= 0; iX < m_iSizeX; ++iX )
		{
			if( get( iX, iY ).m_iPathStep < 0 )
			{
				int iStep= -get( iX, iY ).m_iPathStep - 1;
				if( iStep >= (int)bmNew.m_vecptPos.size() )
				{
					bmNew.m_vecptPos.resize( iStep + 1 );
				}
				bmNew.m_vecptPos[ iStep ]= Point( iX, iY );
			}
		}
	}
	bmNew.m_iColor= bfCopy.m_iColor;
	m_pIngame->m_listBallMoves.push_back( bmNew );

	set( ptStart.m_iX, ptStart.m_iY, BoardField() );
	set( ptEnd.m_iX, ptEnd.m_iY, bfCopy );

	return chain( ptEnd, m_iMinChain, (float)bmNew.m_vecptPos.size() / (float)m_iMoveSpeed );
}

int Ingame::Board::chain( Point ptStart, int iMinCount, float fTimeOffset )
{
	switch( m_pIngame->m_iRules )
	{
	case 1:
		return tobias( ptStart, iMinCount, fTimeOffset );
	}
	return realchain( ptStart, iMinCount, fTimeOffset );
}

int Ingame::Board::realchain( Point ptStart, int iMinCount, float fTimeOffset )
{
	BoardField bfAct= get( ptStart.m_iX, ptStart.m_iY );
	if( bfAct.m_iColor == -1 || bfAct.m_iAppeared < 0)
	{
		return 0;
	}

	if( iMinCount > 0 )
	{
		int iCount= 0;
		for( int iY= -1; iY < 2; ++iY )
		{
			for( int iX= -1; iX < 2; ++iX )
			{
				if( iX == 0 && iY == 0 )
				{
					continue;
				}
				if( get( ptStart.m_iX + iX, ptStart.m_iY + iY ).m_iColor == bfAct.m_iColor &&
					get( ptStart.m_iX + iX, ptStart.m_iY + iY ).m_iAppeared >= 0 )
				{
					iCount++;
				}
			}
		}
		if( iCount < iMinCount )
		{
			return 0;
		}
	}

	float fTime= 1.25f + fTimeOffset;
	std::list< Point > liExploded;

	liExploded.push_back( Point( ptStart.m_iX, ptStart.m_iY ) );

	set( ptStart.m_iX, ptStart.m_iY, BoardField() );
	int iDeleted= 1;

	BallEffect beNew;
	beNew.m_fTime= fTime;
	beNew.m_iColor= bfAct.m_iColor;
	beNew.m_Mode= BallEffect::BE_EXPLODE;
	beNew.m_ptPos= ptStart;

	m_pIngame->m_listBalleffects.push_front( beNew );

	std::list< Point > liNewExploded;

	while( liExploded.size() > 0 )
	{
		fTime+= 0.4f;
		for( std::list< Point>::iterator it= liExploded.begin();
			it != liExploded.end();
			++it )
		{
			for( int iY= -1; iY < 2; ++iY )
			{
				for( int iX= -1; iX < 2; ++iX )
				{
					if( iX == 0 && iY == 0 )
					{
						continue;
					}
					int iNX= it->m_iX + iX;
					int iNY= it->m_iY + iY;
					if( get( iNX, iNY ).m_iColor == bfAct.m_iColor &&
						get( iNX, iNY ).m_iAppeared >= 0 )
					{

						liNewExploded.push_back( Point( iNX, iNY ) );

						set( iNX, iNY, BoardField() );
						iDeleted++;

						BallEffect beNew;
						beNew.m_fTime= fTime;
						beNew.m_iColor= bfAct.m_iColor;
						beNew.m_Mode= BallEffect::BE_EXPLODE;
						beNew.m_ptPos= Point( iNX, iNY );

						m_pIngame->m_listBalleffects.push_front( beNew );
					}
				}
			}
		}
		liExploded= liNewExploded;
		liNewExploded.clear();
	}

	return iDeleted;
}

int Ingame::Board::tobias( Point ptStart, int iMinCount, float fTimeOffset )
{
	BoardField bfAct= get( ptStart.m_iX, ptStart.m_iY );
	if( bfAct.m_iColor == -1 || bfAct.m_iAppeared < 0)
	{
		return 0;
	}

	bool bBoom[3][3];
	for( int iY= 0; iY < 3; ++iY )
	{
		for( int iX= 0; iX < 3; ++iX )
		{
			bBoom[ iY ][ iX ]= false;
		}
	}

	int iDeleted= 0;

	for( int iY= 0; iY < 2; ++iY )
	{
		for( int iX= 0; iX < 2; ++iX )
		{
			bool bAll= true;
			for( int iYOff= 0; iYOff < 2; ++iYOff )
			{
				for( int iXOff= 0; iXOff < 2; ++iXOff )
				{
					int iNX= ptStart.m_iX - 1 + iX + iXOff;
					int iNY= ptStart.m_iY - 1 + iY + iYOff;

					if( get( iNX, iNY ).m_iColor != bfAct.m_iColor ||
						get( iNX, iNY ).m_iAppeared < 0 )
					{
						bAll= false;
					}
				}
			}

			if( bAll )
			{
				iDeleted+= 4;
				for( int iYOff= 0; iYOff < 2; ++iYOff )
				{
					for( int iXOff= 0; iXOff < 2; ++iXOff )
					{
						bBoom[ iX + iXOff ][ iY + iYOff ] = true;
					}
				}
			}
		}
	}

	if( iDeleted > 0 )
	{
		for( int iY= 0; iY < 3; ++iY )
		{
			for( int iX= 0; iX < 3; ++iX )
			{
				if( !bBoom[ iX ][ iY ] )
				{
					continue;
				}

				int iNX= ptStart.m_iX - 1 + iX;
				int iNY= ptStart.m_iY - 1 + iY;			

				set( iNX, iNY, BoardField() );

				BallEffect beNew;
				beNew.m_fTime= fTimeOffset + 1.25f;
				beNew.m_iColor= bfAct.m_iColor;
				beNew.m_Mode= BallEffect::BE_EXPLODE;
				beNew.m_ptPos= Point( iNX, iNY );

				m_pIngame->m_listBalleffects.push_front( beNew );
			}
		}
	}

	return iDeleted;
}

bool Ingame::Board::checkPlayability()
{
	int iBalls= 0;
	int iFree= 0;
	
	do
	{
		for( int iY= 0; iY < m_iSizeY; ++iY )
		{
			for( int iX= 0; iX < m_iSizeX; ++iX )
			{
				if( isFree( iX, iY ) )
				{
					iFree++;
				}
				else
				{
					iBalls++;
				}
			}
		}

		if( iBalls == 0 )
		{
			appear();
			addBalls(	m_pIngame->getBallCount(),
						m_pIngame->getColorCount() );
		}
	}
	while( iBalls == 0 );

	return iFree > 1;
}

int Ingame::getBallCount()
{
	if( m_iLevel < 6 )
	{
		return m_iLevel / 3 + 3;
	}
	else
	{
		return m_iLevel;
	}

}

int Ingame::getColorCount()
{
	return min( m_iMaxColors, m_iLevel + 3 );
}

bool Ingame::getBoardMouseHit( int& iX, int& iY )
{
	FrameWork::vec3 vec3Pos= m_Mouse.getPos();

	float fX= vec3Pos.x;
	fX-= m_fGameBoardX - m_fGameBoardStepX / 2;
	if( m_fGameBoardStepX > 0.01f )
	{
		fX/= m_fGameBoardStepX;
	}
	float fY= vec3Pos.y;
	fY-= m_fGameBoardY - m_fGameBoardStepY / 2;
	if( m_fGameBoardStepY > 0.01f )
	{
		fY/= m_fGameBoardStepY;
	}

	if( fX < 0 )
	{
		fX-= 1.0f;
	}
	if( fY < 0 )
	{
		fY-= 1.0f;
	}

	iX= (int)fX;
	iY= (int)fY;

	return (	iX >= 0 && iX < m_iBoardLengthX &&
				iY >= 0 && iY < m_iBoardLengthY );
}

void Ingame::score( int iDeleted, Point ptPos )
{
	m_iInRow++;
	if( iDeleted > 4 && m_iRules == 1 )
	{
		m_iInRow++;
	}
	int Scored= iDeleted * ( 5 + m_iInRow * 5 );
	m_iDeletedBalls+= iDeleted;
	m_iLevel= m_iDeletedBalls / m_iDeletedPerLevel;
	m_iScore+= Scored;

	std::ostringstream oss;
	if( m_iInRow > 1 )
	{
		oss << m_iInRow <<"x Combo\n";
	}
	oss << Scored << " Points";

	TextEffect teNew;
	teNew.m_vec2Pos= FrameWork::vec2(	m_fGameBoardX + ptPos.m_iX * m_fGameBoardStepX,
										m_fGameBoardY + ptPos.m_iY * m_fGameBoardStepY );
	teNew.m_strText= oss.str();

	m_listTexteffects.push_back( teNew );
}

/************************************************************************************
**	Ende der Datei:		Ingame.cpp
************************************************************************************/
