/***********************************************************************************/
/**	\file				SimpleGame.cpp
**	\brief				Implementation zur Klasse SimpleGame
*************************************************************************************
**	Autor:				Christian Roesch
*************************************************************************************
**	-tut nichts-
**
*//*********************************************************************************/


// includes
#include "SimpleGame.h"
#include "..\ScreenModes.h"

#include "SimpleMenu.h"
#include "Ingame.h"
#include "Outro.h"

// Methoden-Definitionen

SimpleGame::SimpleGame()
:
m_pGame( NULL ),
m_pSimpleMenu( NULL ),
m_pgsIngame( NULL ),
m_pgsOutro( NULL)
{
	m_strOptionsFile= "options.txt";
	m_strSettingsFile= "settings.txt";

	new FrameWork::ScreenModes();
	if( !FrameWork::ScreenModes::inst().read() )
	{
		MessageBox( NULL, "Missing files, please reinstall application.", "Error", MB_OK | MB_ICONEXCLAMATION);
		return;
	}

	//std-Config
	m_cfgOptions.addFloat( "SCREENMODE", 0 );
	m_cfgOptions.addFloat( "COLORS", 32 );
	m_cfgOptions.addFloat( "FULLSCREEN", 0 );
	m_cfgOptions.addFloat( "VSYNC", 1 );

	m_cfgOptions.read( m_strOptionsFile );

	FrameWork::ScreenModes::inst().setActMode( (int)m_cfgOptions.getFloat( "SCREENMODE" ) );

	m_cfgSettings.read( m_strSettingsFile );

	m_pSimpleMenu= new SimpleMenu( &m_cfgOptions, &m_cfgSettings );
	m_pgsIngame= new Ingame( &m_cfgSettings );
	m_pgsOutro= new Outro();

	m_pGame= new FrameWork::Game();

	m_pGame->registerGameState( m_pSimpleMenu );
	m_pGame->registerGameState( m_pgsIngame );
	m_pGame->registerGameState( m_pgsOutro );

	main();
}

SimpleGame::~SimpleGame()
{
	m_cfgSettings.write( m_strSettingsFile );

	delete m_pGame;
	m_pGame= NULL;

	delete m_pSimpleMenu;
	m_pSimpleMenu= NULL;

	delete m_pgsIngame;
	m_pgsIngame= NULL;

	delete m_pgsOutro;
	m_pgsOutro= NULL;
}

void SimpleGame::main()
{
	while( m_pSimpleMenu->isRestart() )
	{
		startGame();

		m_pGame->setGameState( "Menu" );
		m_pGame->run();

		quitGame();
	}
}

void SimpleGame::startGame()
{
	FrameWork::ScreenModes::inst().setActMode( (int)m_cfgOptions.getFloat( "SCREENMODE" ) );

	int iX= FrameWork::ScreenModes::inst().getActMode()->m_iWidth;
	int iY= FrameWork::ScreenModes::inst().getActMode()->m_iHeight;

	int iColors= (int)m_cfgOptions.getFloat( "COLORS" );

	int iFlags= 0;
	if( m_cfgOptions.getFloat( "FULLSCREEN" ) == 0 || 
		!FrameWork::ScreenModes::inst().getActMode()->m_bFullScreen )
	{
		iFlags|= FrameWork::GF_WINDOWED;
		iColors= -1;
	}

	if( m_cfgOptions.getFloat( "VSYNC" ) != 0 )
	{
		iFlags|= FrameWork::GF_VSYNC;
	}

	// Grafikkomponente initialisieren
	new FrameWork::Graphics(	iX, 
						iY,				// Breite und Hhe des Fensters
						iColors,			// Bits pro Pixel (16, 24 oder 32)
						iFlags,			// Flags (z.B. GF_WINDOWED fr Fenstermodus)
						"InChains");	// Name des Programms (erscheint in der Titelleiste)

	// Audiokomponente initialisieren
	new FrameWork::Audio;
	// Eingabekomponente initialisieren
	new FrameWork::Input;

	//alles geklappt, neue gueltige Optionen schreiben
	m_cfgOptions.write( m_strOptionsFile );

	while( ShowCursor( FALSE ) >= 0 );
}

void SimpleGame::quitGame()
{
	while( ShowCursor( TRUE ) < 0 );
	
	// Framework herunterfahren
	FrameWork::Graphics::shutdown();
	FrameWork::Audio::shutdown();
	FrameWork::Input::shutdown();
}

/************************************************************************************
**	Ende der Datei:		SimpleGame.cpp
************************************************************************************/
