/***********************************************************************************/
/**	\file				SimpleMenu.cpp
**	\brief				Implementation zur Klasse SimpleMenu
*************************************************************************************
**	Autor:				Christian Roesch
*************************************************************************************
**	-tut nichts-
**
*//*********************************************************************************/


// includes
#include "SimpleMenu.h"
#include "../framework.h"
#include "../ScreenModes.h"
#include <sstream>

// Methoden-Definitionen

SimpleMenu::SimpleMenu( FrameWork::Config* pConfig,
						FrameWork::Config* pSettings)
:
m_pFont1( NULL ),
m_pFont2( NULL ),
m_eActiveMenu( MENU_MAIN ),
m_iActiveChoice( 0 ),
m_bRestart( true ),
m_pConfig( pConfig ),
m_pSettings( pSettings )
{
	m_vecstrRules.push_back( "Chains" );
	m_vecstrRules.push_back( "Tobias" );
}

SimpleMenu::~SimpleMenu()
{
	delete m_pFont1;
	m_pFont1= NULL;
	delete m_pFont2;
	m_pFont2= NULL;
}

std::string SimpleMenu::getName() const 
{
	return "Menu";
}

void SimpleMenu::onEnter( void* p_context )
{
	FrameWork::ScreenModes::Mode* pMode= FrameWork::ScreenModes::inst().getActMode();

	m_fScreenX= (float)pMode->m_iWidth;
	m_fScreenY= (float)pMode->m_iHeight;
	
	m_fArrowX= pMode->getScaledFloatX( "ARROWX" );
	m_fArrowY= pMode->getScaledFloatY( "ARROWY" );
	m_fMenuHeadX= pMode->getScaledFloatX( "MENUHEADX" );
	m_fMenuHeadY= pMode->getScaledFloatY( "MENUHEADY" );
	m_fMenuHeadSizeX= pMode->getScaledFloatX( "MENUHEADSIZEX" );
	m_fMenuHeadSizeY= pMode->getScaledFloatY( "MENUHEADSIZEY" );

	m_fMenuFirstX= pMode->getScaledFloatX( "MENUFIRSTX" );
	m_fMenuFirstY= pMode->getScaledFloatY( "MENUFIRSTY" );
	m_fMenuStep= pMode->getScaledFloatY( "MENUSTEP" );

	m_fMenuSizeX= pMode->getScaledFloatX( "MENUSIZEX" );
	m_fMenuSizeY= pMode->getScaledFloatY( "MENUSIZEY" );

	m_Mouse.setClipRect( 0, m_fScreenX * 0.98f, 0, m_fScreenY * 0.98f );
	m_Mouse.setPos( FrameWork::vec3( m_fScreenX * 0.5f, m_fScreenY * 0.5f, 0 ) );

	m_pFont1=	new FrameWork::Font(
						pMode->m_cfgVirtualData.getString( "MENUFONT1" ),
						(int)pMode->getScaledFloatX( "MENUFONT1_X" ), 
						(int)pMode->getScaledFloatY( "MENUFONT1_Y" ),
						FrameWork::FF_BOLD );
	m_pFont2=	new FrameWork::Font(
						pMode->m_cfgVirtualData.getString( "MENUFONT2" ),
						(int)pMode->getScaledFloatX( "MENUFONT2_X" ), 
						(int)pMode->getScaledFloatY( "MENUFONT2_Y" ),
						0 );

	m_csMenu.load( "gfx/menu.txt" );
	m_csArrow.load( "gfx/mouse.txt" );
}

void SimpleMenu::onLeave( void* p_context, bool quit )
{
	delete m_pFont1;
	m_pFont1= NULL;
	delete m_pFont2;
	m_pFont2= NULL;

	m_csMenu.free();
	m_csArrow.free();
}

void SimpleMenu::render( float dt )
{
	FrameWork::Graphics& graphics = FrameWork::Graphics::inst();
	FrameWork::Surface* pBackBuffer= graphics.getBackBuffer();
	// Back-Buffer leeren (Farbe: R, G, B, A)
	pBackBuffer->clear(FrameWork::color(0.0f, 0.0f, 0.0f, 1.0f));

	int iCount= getMenuLength( m_eActiveMenu );

	for( int i= 0; i < iCount; ++i )
	{
		int iFrame= ( i == m_iActiveChoice ) ? 1 : 0;
		m_csMenu.drawFrame(	pBackBuffer,
								FrameWork::vec2( m_fMenuFirstX, m_fMenuFirstY + m_fMenuStep * (float)i ),
								FrameWork::vec2( m_fMenuSizeX, m_fMenuSizeY ),
								iFrame );

		std::string strFirst;
		std::string strSecond;

		getMenuTexts( strFirst, strSecond, m_eActiveMenu, i );

		DWORD dwColorFirst= isActiveMenu( m_eActiveMenu, i ) ? 0xffffffff : 0xa0808080;
		DWORD dwColorSecond= ( i == m_iActiveChoice ) ? 0xffffffff : 0xa0808080;
		dwColorSecond= isActiveMenu( m_eActiveMenu, i ) ? dwColorSecond : 0x00000000;

		m_pFont1->drawText(	pBackBuffer,
							FrameWork::vec2( m_fMenuFirstX, m_fMenuFirstY + m_fMenuStep * (float)i ),
							strFirst,
							FrameWork::DTF_HCENTER|FrameWork::DTF_VCENTER,
							dwColorFirst );
		m_pFont2->drawText(	pBackBuffer,
							FrameWork::vec2( m_fMenuFirstX + m_fMenuSizeX * 0.75f, m_fMenuFirstY + m_fMenuStep * (float)i ),
							strSecond,
							FrameWork::DTF_VCENTER,
							dwColorSecond );
	}

	FrameWork::vec3 vec3Pos= m_Mouse.getPos();
	m_csArrow.drawFrame(	pBackBuffer,
							FrameWork::vec2( vec3Pos.x + m_fArrowX / 2, vec3Pos.y + m_fArrowY / 2 ),
							FrameWork::vec2( m_fArrowX, m_fArrowY ),
							0 );


	graphics.showBackBuffer( 1.0f );
}

void SimpleMenu::update( float dt )
{
	m_Mouse.update();
	FrameWork::Input& input = FrameWork::Input::inst();

	if( input.wasKeyboardKeyPressed( DIK_ESCAPE ) )
	{
		activateMenu( m_eActiveMenu, -1 );
	}

	if( input.wasKeyboardKeyPressed( DIK_RETURN ) )
	{
		activateMenu( m_eActiveMenu, m_iActiveChoice );
	}

	if( input.wasKeyboardKeyPressed( DIK_DOWNARROW ) )
	{
		walkSelection( 1 );
	}

	if( input.wasKeyboardKeyPressed( DIK_UPARROW ) )
	{
		walkSelection( -1 );
	}

	if( m_Mouse.isMouseMove() )
	{
		int iSelection= findMouseSelection();
		if( iSelection >= 0 )
		{
			changeSelection( iSelection );
		}
	}

	if( m_Mouse.isButtonClick( 0 ) )
	{
		int iSelection= findMouseSelection();
		if( iSelection >= 0 )
		{
			activateMenu( m_eActiveMenu, iSelection );
		}
	}
}

int SimpleMenu::getMenuLength( Menus eMenu )
{
	switch( eMenu )
	{
	case MENU_MAIN:
		return 5;
	case MENU_GFXOPTIONS:
		return 5;
	case MENU_CREDITS:
		return 4;
	}
	
	return 0;
}


void SimpleMenu::getMenuTexts(	std::string& strFirst, 
								std::string& strSecond,
								Menus eMenu,
								int iIndex )
{
	int iMode= (int)m_pConfig->getFloat( "SCREENMODE" );
	FrameWork::ScreenModes::Mode* pMode= 
		FrameWork::ScreenModes::inst().getMode( iMode );
	switch( eMenu )
	{
	case MENU_MAIN:
		switch( iIndex )
		{
		case 0:
			strFirst= "Start";
			strSecond= "start a new game";
			return;
		case 1:
			{
				strFirst= "Rules";
				int iValue= (int)m_pSettings->getFloat( "RULES" );
				strSecond= m_vecstrRules[ iValue ];
				return;
			}
		case 2:
			strFirst= "Options";
			strSecond= "set preferences";
			return;
		case 3:
			strFirst= "Credits";
			strSecond= "we build this crap";
			return;
		case 4:
			strFirst= "Quit";
			strSecond= "leave game";
			return;
		}
		break;
	case MENU_GFXOPTIONS:
		switch( iIndex )
		{
		case 0:
			strFirst= "Screen Mode";
			strSecond= pMode->m_strName;
			return;
		case 1:
			strFirst= "Colors";
			{
				std::ostringstream oss;
				oss	<< (int)m_pConfig->getFloat( "COLORS" )
					<< " bit";
				strSecond= oss.str();
			}
			return;
		case 2:
			strFirst= "Fullsreen";
			{
				std::ostringstream oss;
				oss	<< ( (int)m_pConfig->getFloat( "FULLSCREEN" ) ? "enabled" : "disabled");
				strSecond= oss.str();
			}
			return;
		case 3:
			strFirst= "V-Sync";
			{
				std::ostringstream oss;
				oss	<< ( (int)m_pConfig->getFloat( "VSYNC" ) ? "enabled" : "disabled" );
				strSecond= oss.str();
			}
			return;
		case 4:
			strFirst= "Apply";
			strSecond= "apply changes";
			return;
		}
		break;
	case MENU_CREDITS:
		switch( iIndex )
		{
		case 0:
			strFirst= "Game Code";
			strSecond= "Christian Rsch";
			return;
		case 1:
			strFirst= "Framework Code";
			strSecond= "David Scherfgen\nChristian Rsch";
			return;
		case 2:
			strFirst= "useless GFX";
			strSecond= "Christian Rsch";
			return;
		case 3:
			strFirst= "Back";
			strSecond= "to Main Menu";
			return;
		}
		break;
	}
	
	strFirst= "";
	strSecond= "";
	return;
}

bool SimpleMenu::isActiveMenu(	Menus eMenu,
					int iIndex )
{
	switch( eMenu )
	{
	case MENU_GFXOPTIONS:
		{
			int iMode= (int)m_pConfig->getFloat( "SCREENMODE" );
			FrameWork::ScreenModes::Mode* pMode= 
				FrameWork::ScreenModes::inst().getMode( iMode );
			switch( iIndex )
			{
			case 2:
				return pMode->m_bFullScreen;
			}
		}
		break;
	case MENU_CREDITS:
		break;
	}

	return ( iIndex >= 0 && iIndex < getMenuLength( eMenu ) ); 
}

void SimpleMenu::activateMenu(	Menus eMenu,
								int iIndex )
{
	switch( eMenu )
	{
	case MENU_MAIN:
		switch( iIndex )
		{
		case -1:
			m_bRestart= false;
			getGame()->setGameState( "Outro" );
			return;
		case 0:
			getGame()->setGameState( "Ingame" );
			return;
		case 1:
			{
				int iValue= (int)m_pSettings->getFloat( "RULES" );
				iValue= 1 - iValue;
				if( iValue != 1 && iValue != 0 )
				{
					iValue= 0;
				}
				m_pSettings->addFloat( "RULES", (float)iValue );
			}
			return;
		case 2:
			changeMenu( MENU_GFXOPTIONS );
			return;
		case 3:
			changeMenu( MENU_CREDITS );
			return;
		case 4:
			m_bRestart= false;
			getGame()->setGameState( "Outro" );
			return;
		}
		break;
	case MENU_GFXOPTIONS:
		activateGfxOption( iIndex );
		break;
	case MENU_CREDITS:
		changeMenu( MENU_MAIN );
		break;
	}
	
	return;

}

void SimpleMenu::changeMenu( Menus eNewMenu )
{
	m_eActiveMenu= eNewMenu;
	m_iActiveChoice= 0;
	findActiveChoice();
}

void SimpleMenu::findActiveChoice( int iDir/*= 1*/ )
{
	int iBegin= m_iActiveChoice;

	iDir= iDir % getMenuLength( m_eActiveMenu );
	if( iDir < 0 )
	{
		iDir+= getMenuLength( m_eActiveMenu );
	}

	while( !isActiveMenu( m_eActiveMenu, m_iActiveChoice ) )
	{
		m_iActiveChoice+= iDir;
		m_iActiveChoice= m_iActiveChoice % getMenuLength( m_eActiveMenu );

		if( m_iActiveChoice == iBegin )
		{
			//nichts gefunden
			m_iActiveChoice= -1;
			return;
		}
	}
}

void SimpleMenu::changeSelection( int iNew )
{
	if( isActiveMenu( m_eActiveMenu, iNew ) )
	{
		m_iActiveChoice= iNew;
	}
}

void SimpleMenu::walkSelection( int iDir )
{
	iDir= iDir % getMenuLength( m_eActiveMenu );
	if( iDir < 0 )
	{
		iDir+= getMenuLength( m_eActiveMenu );
	}

	m_iActiveChoice+= iDir;
	m_iActiveChoice= m_iActiveChoice % getMenuLength( m_eActiveMenu );

	findActiveChoice( iDir );
}

bool SimpleMenu::isRestart()
{
	return m_bRestart;
}

void SimpleMenu::activateGfxOption( int iIndex )
{
	int iMode= (int)m_pConfig->getFloat( "SCREENMODE" );
	FrameWork::ScreenModes::Mode* pMode= 
			FrameWork::ScreenModes::inst().getMode( iMode );

	switch( iIndex )
	{
	case -1:
		changeMenu( MENU_MAIN );
		return;
	case 0:
		{
			iMode++;
			if( iMode >= FrameWork::ScreenModes::inst().getModeCount() )
			{
				iMode= 0;
			}
			m_pConfig->addFloat( "SCREENMODE", (float)iMode );
		}
		return;
	case 1:
		{
			int iColor= (int)m_pConfig->getFloat( "COLORS" );
			iColor= 48 - iColor;
			if( iColor != 32 && iColor != 16 )
			{
				iColor= 16;
			}
			m_pConfig->addFloat( "COLORS", (float)iColor );
		}
		return;
	case 2:
		{
			int iValue= (int)m_pConfig->getFloat( "FULLSCREEN" );
			iValue= 1 - iValue;
			if( iValue != 1 && iValue != 0 )
			{
				iValue= 0;
			}
			m_pConfig->addFloat( "FULLSCREEN", (float)iValue );
		}
		return;
	case 3:
		{
			int iValue= (int)m_pConfig->getFloat( "VSYNC" );
			iValue= 1 - iValue;
			if( iValue != 1 && iValue != 0 )
			{
				iValue= 0;
			}
			m_pConfig->addFloat( "VSYNC", (float)iValue );
		}
		return;
	case 4:
		changeMenu( MENU_MAIN );
		m_bRestart= true;
		getGame()->quit();
		return;
	}
}

int SimpleMenu::findMouseSelection()
{
	FrameWork::vec3 vec3Pos= m_Mouse.getPos();

	for( int i= 0; i < getMenuLength( m_eActiveMenu ); ++i )
	{
		if( isActiveMenu( m_eActiveMenu, i ) &&
			abs( m_fMenuFirstX - vec3Pos.x ) < m_fMenuSizeX / 2 &&
			abs( m_fMenuFirstY + m_fMenuStep * (float)i - vec3Pos.y ) < m_fMenuSizeY / 2 )
		{
			return i;
		}
	}

	return -1;
}

/************************************************************************************
**	Ende der Datei:		SimpleMenu.cpp
************************************************************************************/
