#include "SimpleGame.h"

#include "../framework.h"
using namespace std;
using namespace FrameWork;

// eigene abgeleitete Spielklasse
class MyGame : public Game
{
public:
	MyGame( int x, int y )
	{
		// Grafikkomponente initialisieren
		new Graphics(x, y,		// Breite und Hhe des Fensters
					 32,			// Bits pro Pixel (16, 24 oder 32)
					 GF_VSYNC,		// Flags (z.B. GF_WINDOWED fr Fenstermodus)
					 "Programm");	// Name des Programms (erscheint in der Titelleiste)

		// Audiokomponente initialisieren
		new Audio;

		// Eingabekomponente initialisieren
		new Input;
	}

	~MyGame()
	{
		// Framework herunterfahren
		Graphics::shutdown();
		Audio::shutdown();
		Input::shutdown();
	}
};

// eigene abgeleitete Spielzustandsklasse fr Intro
class Intro : public GameState
{
public:
	Intro()
	{
		// Timer anlegen
		p_timer = new Timer;

		// Sprite laden
		p_dusmaniaLogo = new Surface("dusmania.bmp",				// Dateiname
									 true,							// Color-Key benutzen?
									 D3DCOLOR_XRGB(255, 0, 255));	// Color-Key (Pink)

		// Bild mit Animations-Frames laden
		p_player = new Surface("animation.bmp", true, D3DCOLOR_XRGB(0, 0, 0));

		// ein neues Animations-Set erzeugen, welches spter die beiden Animationen fr
		// das Laufen nach rechts und nach links enthalten wird
		p_animSet = new AnimationSet;

		// Animation "walk right" erzeugen
		Animation* p_walkRight = p_animSet->createAnimation("walk right",	// Name
															15.0f);			// Geschwindigkeit (fps)

		// Automatisch zwlf Frames fr diese Animation generieren lassen.
		// Diese Frames beginnen an Position (0, 0) in der Datei "animation.bmp".
		// Jedes Frame ist 40x80 Pixel gro, wobei noch ein 1 Pixel breiter weier Rand
		// hinzukommt, der nicht bercksichtigt werden soll.
		p_walkRight->generateFrames(12,								// Anzahl der Frames
									ivec2(0, 0),					// linke obere Ecke des 1. Frames
									ivec2(40, 80),					// genutzte Gre eines Frames
									ivec2(41, 81),					// gesamter Bereich fr ein Frame
									0,								// Randpixel auf der linken Seite bei "Zeilenumbruch"
									p_player->getSurfaceSize());	// Gre der Oberflche mit den Frames

		// entsprechend verfahren fr die andere Animation
		Animation* p_walkLeft = p_animSet->createAnimation("walk left", 15.0f);
		p_walkLeft->generateFrames(12, ivec2(0, 81), ivec2(40, 80), ivec2(41, 81), 0, p_player->getSurfaceSize());

		// animiertes Objekt erzeugen, das sich dem zuvor erzeugten Animation-Set bedient
		p_playerAnim = new Animated(p_animSet);

		// Schriftart erzeugen
		p_smallFont = new Font("Arial",			// Name der Schriftart
							   10, 25,			// Breite und Hhe eines Zeichens
							   FF_ULTRABOLD);	// Flags (z.B. FF_ITALIC fr kursiv)

		// Sound laden
		p_explosionSound = new Sound("explosion.wav");
	}

	~Intro()
	{
		// Ressourcen freigeben
		safeDelete(p_timer);
		safeDelete(p_dusmaniaLogo);
		safeDelete(p_player);
		safeDelete(p_animSet);
		safeDelete(p_playerAnim);
		safeDelete(p_smallFont);
		safeDelete(p_explosionSound);
	}

	// Diese Methode liefert den Namen des Spielzustands. Sehr wichtig!
	std::string getName() const {return "Intro";}

	void onEnter(void* p_context)
	{
		// Diese Methode wird beim Betreten des Spielzustands aufgerufen.

		// Startposition des Sprites festlegen
		spritePosition = vec2(400.0f, 300.0f);

		// Startzustand der Figur setzen
		playerPosition = vec2(400.0f, 500.0f);
		p_playerAnim->setAnimation("walk left");
		p_playerAnim->play();
	}

	void onLeave(void* p_context,
				 bool quit)
	{
		// Diese Methode wird beim Verlassen des Spielzustands aufgerufen.
		// Wird der Spielzustand verlassen, weil das Spiel beendet wurde, dann hat
		// der Parameter quit den Wert true.
	}

	void render(float dt)
	{
		/* TO DO: Hier die Objekte des Spiels rendern */

		// Cursor ausblenden
		//ShowCursor(FALSE);

		// Graphics-Singleton, Back-Buffer und Wert des Timers holen
		Graphics& graphics = Graphics::inst();
		Surface* p_backBuffer = graphics.getBackBuffer();
		const float t = *p_timer;

		static float fTest= 0;
		fTest+= 0.01f;

		// Back-Buffer leeren (Farbe: R, G, B, A)
		p_backBuffer->clear(color(0.0f, 0.0f, 0.5f, 1.0f));

		// einfaches rotierendes Sprite zeichnen
		p_backBuffer->drawSprite(p_dusmaniaLogo,					// Sprite
								 vec2(0.0f + fTest, 200.0f),					// Position
								 vec2(820.0f, 230.0f),				// Gre
								 0,									// Rotation (in rad)
								 color(1.0f, 1.0f, 1.0f, 1.0f));	// Farbe (R, G, B, A)

		// Das animierte Sprite zeichnen.
		// Dazu holen wir uns die Daten des aktuellen Frames der Animation
		// und benutzen sie als Parameter fr die Methode drawSubSprite, um den richtigen Teil
		// der Oberflche mit allen Animationsframes darzustellen.
		const AnimationFrame& frame = p_playerAnim->getCurrentFrame();
		p_backBuffer->drawSubSprite(p_player, frame.coords, frame.size, playerPosition, frame.size);


		// kleinen Text zur Demonstration ausgeben
		p_smallFont->drawText(p_backBuffer,						// Zieloberflche
							  vec2(50.0f, 50.0f),				// Position
							  "Maus bewegt das Logo.\nPfeiltasten bewegen die Figur.\nLeertaste spielt einen Sound ab.\nEscape drcken, um das Programm zu beenden!",
							  0,								// Flags (z.B. DTF_HCENTER zum horizontalen Zentrieren)
							  color(1.0f, 1.0f, 1.0f, 1.0f));	// Farbe (R, G, B, A)

		// Inhalt des Back-Buffers sichtbar machen
		// (Parameter bestimmt die Transparenz - kleine Werte fhren zu Motion-Blur!
		//  Zu kleine Werte fhren zu "Farbflecken")
		graphics.showBackBuffer(0.4f);
	}

	void update(float dt)
	{
		/* TO DO: Hier die Objekte des Spiels bewegen, auf Benutzereingaben reagieren etc. */

		// Input-Singleton holen
		Input& input = Input::inst();

		// Bewegung der Figur durch Pfeiltasten (120 Pixel pro Sekunde)
		if(input.isKeyboardKeyDown(DIK_LEFT))
		{
			// Bewegung
			playerPosition.x -= dt * 120.0f;

			// Solange diese Taste gedrckt ist, wird die Animation "walk left" abgespielt.
			p_playerAnim->resume();
			p_playerAnim->setAnimation("walk left");
		}
		else if(input.isKeyboardKeyDown(DIK_RIGHT))
		{
			playerPosition.x += dt * 120.0f;
			p_playerAnim->resume();
			p_playerAnim->setAnimation("walk right");
		}
		else
		{
			// Wenn keine Taste gedrckt wird, soll das erste Frame der Animation
			// angezeigt werden.
			p_playerAnim->resetAnimation();
		}

		// Bewegung durch die Maus
		int mx = 0, my = 0;
		input.getMouseMovement(&mx, &my, 0);
		spritePosition.x += static_cast<float>(mx);
		spritePosition.y += static_cast<float>(my);

		// Wenn die Leertaste gedrckt wurde, wird ein Sound abgespielt.
		if(input.wasKeyboardKeyPressed(DIK_SPACE))
		{
			// Instanz des Sounds erstellen.
			// Der bool-Parameter gibt an, ob diese Instanz auch noch weiterhin bestehen
			// bleiben soll, nachdem sie fertig abgespielt wurde. Dies ist normalerweise
			// nicht notwendig.
			SoundInstance* p_instance = p_explosionSound->createInstance(false);

			// zufllige Abspielfrequenz setzen
			p_instance->setParameters(SPF_FREQUENCY,			// Was soll gesetzt werden?
									  0,						// Lautstrke
									  0,						// Balance
									  intRand(10000, 20000));	// Frequenz

			// Die Sound-Instanz abspielen.
			// Der bool-Parameter gibt an, ob der Sound in einer Endlosschleife
			// gespielt werden soll.
			p_instance->play(false);
		}

		// Wenn Escape gedrckt wurde, wird das Spiel beendet.
		if(input.wasKeyboardKeyPressed(DIK_ESCAPE)) getGame()->quit();

		// Hinweis:
		// Ein Wechsel zu einem anderen Spielzustand (z.B. aus dem Hauptmen heraus)
		// ist wie folgt mglich:
		// getGame()->setGameState("MainMenu");
	}

private:
	Timer* p_timer;				// Timer
	Surface* p_dusmaniaLogo;	// einfaches Sprite
	Surface* p_player;			// Oberflche mit einer Animation
	AnimationSet* p_animSet;	// Animations-Set
	Animated* p_playerAnim;		// animiertes Objekt
	Font* p_smallFont;			// Schriftart
	Sound* p_explosionSound;	// Explosionssound
	vec2 spritePosition;		// Position des Sprites
	vec2 playerPosition;		// Position des Spielers
};

int WINAPI WinMain(HINSTANCE instance,
				   HINSTANCE prevInstance,
				   char* p_cmdLine,
				   int showCmd)
{
	SimpleGame gameObj;
	
	/*
	{
		MyGame game( 800, 600 );

		game.registerGameState(new Intro);
		// game.registerGameState(new MainMenu);
		// game.registerGameState(new InGame);
		// game.registerGameState(new Credits);

		// Startspielzustand setzen
		game.setGameState("Intro");

		// Spiel laufen lassen
		game.run();
	}*/


return 0;
}
