/***********************************************************************************/
/**	\file				Mouse.cpp
**	\brief				Implementation zur Klasse Mouse
*************************************************************************************
**	Autor:				Christian Roesch
*************************************************************************************
**	-tut nichts-
**
*//*********************************************************************************/


// includes
#include "Mouse.h"

#include "Input.h"

namespace FrameWork
{

// Methoden-Definitionen

Mouse::Mouse()
{
	m_vec3Position= D3DXVECTOR3( 512, 320, 0 );

	m_fSensivity= 1.0f;

	m_fLeft= 0.0f;
	m_fRight= 1024.0f;
	m_fTop= 0.0f;
	m_fBottom= 768.0f;
}

Mouse::~Mouse()
{
}

void Mouse::update()
{
	int iX = 0;
	int iY = 0;
	int iZ = 0;

	Input& input = Input::inst();
	input.getMouseMovement( &iX, &iY, &iZ );
	
	m_vec3Position.x+= static_cast<float>( iX ) * m_fSensivity;
	m_vec3Position.y+= static_cast<float>( iY ) * m_fSensivity;
	m_vec3Position.z+= static_cast<float>( iZ ) * m_fSensivity;

	clipPosition();

	m_bMoved= ( iX != 0 || iY != 0 || iZ != 0 );
}

D3DXVECTOR3 Mouse::getPos()
{
	return m_vec3Position;
}

void Mouse::setPos( D3DXVECTOR3 vec3NewPos )
{
	m_vec3Position= vec3NewPos;
}
	
void Mouse::setSensivity( float fSensivity )
{
	m_fSensivity= fSensivity;
}

void Mouse::setClipRect(	float fLeft,
							float fRight,
							float fTop,
							float fBottom )
{
	m_fLeft= fLeft;
	m_fRight= fRight;
	m_fTop= fTop;
	m_fBottom= fBottom;

	clipPosition();
}

int Mouse::isButtonDown( int iButton )
{
	Input& input = Input::inst();
	return input.wasMouseButtonPressed( iButton );
}

int Mouse::isButtonDrag( int iButton )
{
	Input& input = Input::inst();
	return input.isMouseButtonDown( iButton );
}

int Mouse::isButtonClick( int iButton )
{
	Input& input = Input::inst();
	return input.wasMouseButtonReleased( iButton );
}

bool Mouse::isMouseMove()
{
	return m_bMoved;
}

void Mouse::clipPosition()
{
	if( m_vec3Position.x < m_fLeft )
	{
		m_vec3Position.x= m_fLeft;
	}
	if( m_vec3Position.x > m_fRight )
	{
		m_vec3Position.x= m_fRight;
	}
	if( m_vec3Position.y < m_fTop )
	{
		m_vec3Position.y= m_fTop;
	}
	if( m_vec3Position.y > m_fBottom )
	{
		m_vec3Position.y= m_fBottom;
	}
}

}

/************************************************************************************
**	Ende der Datei:		Mouse.cpp
************************************************************************************/
