/*	                     __      __
	                     \ \    / /
	                      \ \  / /
	         _      _______\ \/ /_______      _
	        / \    / __________________ \    / \
	       (   \  | /        __        \ |  /   )
	     ___\  |  | |       / _\       | |  |  /___
	    |____  |  | |      ( <_        | |  | _____|
	      / \| |  | |       \__/       | |  | |/ \
	     / \ | |  | \__________________/ |  | | / \
	    /_/ \__/   \____________________/   \__/ \_\
	 ___  _   _ ___ __  __   _   _  _ ___   _     ____ __
	|   \| | | / __|  \/  | /_\ | \| |_ _| /_\   |__  /  \
	| |) | |_| \__ \ |\/| |/ _ \| .` || | / _ \    / / () |
	|___/ \___/|___/_|  |_/_/ \_\_|\_|___/_/ \_\  /_(_)__/

	      Framework fr den Spieleprogrammierwettbewerb
	             by David Scherfgen (TomasRiker)
*/

#ifndef _ANIMATION_H
#define _ANIMATION_H

#include <string>
#include <vector>
#include "ivec2.h"

namespace FrameWork
{
	// Struktur fr ein Frame einer Animation
	struct AnimationFrame
	{
		ivec2 coords;	// Koordinaten des Frames
		ivec2 size;		// Gre des Frames
	};

	// Klasse fr eine Animation
	class Animation
	{
		friend class AnimationSet;

	public:
		// fgt ein Frame hinzu
		void addFrame(const ivec2& coords, const ivec2& size);

		// generiert Frames automatisch
		void generateFrames(int num, const ivec2& startCoords, const ivec2& framesSize, const ivec2& pitch, int margin, const ivec2& surfaceSize);

		const std::string& getName() const {return name;}
		void setSpeed(float speed) {this->speed = speed;}
		float getSpeed() const {return speed;}
		int getNumFrames() const {return static_cast<int>(frames.size());}
		const AnimationFrame& getFrame(int n) const {return frames[n];}

	private:
		Animation(const std::string& name, float speed);
		~Animation();

		std::string name;
		float speed;
		std::vector<AnimationFrame> frames;
	};
}

#endif
