/*	                     __      __
	                     \ \    / /
	                      \ \  / /
	         _      _______\ \/ /_______      _
	        / \    / __________________ \    / \
	       (   \  | /        __        \ |  /   )
	     ___\  |  | |       / _\       | |  |  /___
	    |____  |  | |      ( <_        | |  | _____|
	      / \| |  | |       \__/       | |  | |/ \
	     / \ | |  | \__________________/ |  | | / \
	    /_/ \__/   \____________________/   \__/ \_\
	 ___  _   _ ___ __  __   _   _  _ ___   _     ____ __
	|   \| | | / __|  \/  | /_\ | \| |_ _| /_\   |__  /  \
	| |) | |_| \__ \ |\/| |/ _ \| .` || | / _ \    / / () |
	|___/ \___/|___/_|  |_/_/ \_\_|\_|___/_/ \_\  /_(_)__/

	      Framework fr den Spieleprogrammierwettbewerb
	             by David Scherfgen (TomasRiker)
*/

#include <cassert>
#include "safe.h"
#include "font.h"
#include "graphics.h"
#include "surface.h"
using namespace std;

namespace FrameWork
{
	Font::Font(const string& fontName,
			   int width,
			   int height,
			   DWORD flags)
	{
		int weight = FW_NORMAL;
		if(flags & FF_BOLD) weight = FW_BOLD;
		if(flags & FF_ULTRABOLD) weight = FW_HEAVY;

		HRESULT hres = D3DXCreateFont(Graphics::inst().getD3DDevice(),
									  height, width,
									  weight,
									  1,
									  (flags & FF_ITALIC) ? TRUE : FALSE,
									  ANSI_CHARSET,
									  OUT_TT_PRECIS,
									  ANTIALIASED_QUALITY,
									  FF_DONTCARE | DEFAULT_PITCH,
									  fontName.c_str(),
									  &p_font);
		assert(SUCCEEDED(hres));
		assert(p_font);

		p_font->GetDesc(&fontDesc);
	}

	Font::~Font()
	{
		safeRelease(p_font);
	}

	void Font::onLostDevice()
	{
		HRESULT hres = p_font->OnLostDevice();
		assert(SUCCEEDED(hres));
	}

	void Font::onResetDevice()
	{
		HRESULT hres = p_font->OnResetDevice();
		assert(SUCCEEDED(hres));
	}

	void Font::drawText(Surface* p_dest,
						const D3DXVECTOR2& position,
						const string& text,
						DWORD flags,
						DWORD color)
	{
		assert(p_dest);

		D3DXVECTOR2 screenPosition;
		if(flags & DTF_USECAMERA) screenPosition = position - Graphics::inst().getCameraPosition();
		else screenPosition = position;

		RECT textSize;
		SetRect(&textSize, 0, 0, 0, 0);
		p_font->DrawText(0, text.c_str(), -1, &textSize, DT_CALCRECT, 0);

		RECT rect;
		rect.left = static_cast<int>(screenPosition.x);
		rect.top = static_cast<int>(screenPosition.y);
		rect.right = rect.left;
		rect.bottom = rect.top;

		if(flags & DTF_HCENTER) rect.left -= textSize.right / 2, rect.right = rect.left + textSize.right;
		else if(flags & DTF_RIGHT) rect.left -= textSize.right;
		if(flags & DTF_VCENTER) rect.top -= textSize.bottom / 2;
		else if(flags & DTF_BOTTOM) rect.top -= textSize.bottom;

		p_dest->beginRender();
		p_dest->flushRenderQueue();

		HRESULT hres = p_font->DrawText(0, text.c_str(), -1, &rect, DT_NOCLIP | ((flags & DTF_HCENTER) ? DT_CENTER : DT_LEFT) | DT_TOP, color);
		assert(SUCCEEDED(hres));
	}
}
