/*	                     __      __
	                     \ \    / /
	                      \ \  / /
	         _      _______\ \/ /_______      _
	        / \    / __________________ \    / \
	       (   \  | /        __        \ |  /   )
	     ___\  |  | |       / _\       | |  |  /___
	    |____  |  | |      ( <_        | |  | _____|
	      / \| |  | |       \__/       | |  | |/ \
	     / \ | |  | \__________________/ |  | | / \
	    /_/ \__/   \____________________/   \__/ \_\
	 ___  _   _ ___ __  __   _   _  _ ___   _     ____ __
	|   \| | | / __|  \/  | /_\ | \| |_ _| /_\   |__  /  \
	| |) | |_| \__ \ |\/| |/ _ \| .` || | / _ \    / / () |
	|___/ \___/|___/_|  |_/_/ \_\_|\_|___/_/ \_\  /_(_)__/

	      Framework fr den Spieleprogrammierwettbewerb
	             by David Scherfgen (TomasRiker)
*/
// modified 12.11.05 (TGGC)

#include <cassert>
#include <windows.h>
#include "game.h"
#include "gamestate.h"
#include "graphics.h"
#include "input.h"
#include "dsoundresource.h"
#include "timer.h"
using namespace std;

namespace FrameWork
{
	Game::Game(bool useFixedLogicStep,
			   float fixedLogicStep) : logicTimeLeft(0.0f),
									   paused(false),
									   p_currentGS(0)
	{
		this->useFixedLogicStep = useFixedLogicStep;
		this->fixedLogicStep = fixedLogicStep;
	}

	Game::~Game()
	{
		// aktuellen Spielzustand verlassen
		if(p_currentGS)
		{
			p_currentGS->onLeave(0, true);
			p_currentGS = 0;
		}

		// Spielzustnde lschen
		// TGGC: Warum sollte diese Klasse pltzlich dafr zustndig sein?
		/*for(map<string, GameState*>::iterator i = gameStates.begin(); i != gameStates.end(); i++)
		{
			delete i->second;
		}*/
	}

	void Game::registerGameState(GameState* p_gs)
	{
		assert(p_gs);
		gameStates[p_gs->getName()] = p_gs;
		p_gs->p_game = this;
	}

	void Game::setGameState(const string& gs,
							void* p_context)
	{
		GameState* p_gs = 0;
		map<string, GameState*>::iterator i = gameStates.find(gs);
		if(i != gameStates.end()) p_gs = i->second;

		// alten Spielzustand herunterfahren
		if(p_currentGS) p_currentGS->onLeave(p_context, false);

		// neuen Spielzustand betreten
		p_currentGS = p_gs;
		if(p_currentGS) p_currentGS->onEnter(p_context);
	}

	void Game::run()
	{
		Graphics& graphics = Graphics::inst();
		Input& input = Input::inst();
		float dt = 0.0f;

		LONGLONG freq;
		QueryPerformanceFrequency(reinterpret_cast<LARGE_INTEGER*>(&freq));
		const float counterFreq = static_cast<float>(freq);

		while(true)
		{
			// Fensternachrichten verarbeiten
			if(!graphics.processWindowMessages())
			{
				// Das Spiel soll beendet werden!
				break;
			}

			// Eingabe verarbeiten
			vector<InputEvent> events = input.processInput();

			LONGLONG startTime;
			QueryPerformanceCounter(reinterpret_cast<LARGE_INTEGER*>(&startTime));

			if(graphics.isWindowActive() && graphics.isLost())
			{
				// Direct3D wiederherstellen
				graphics.tryToRestore();
			}

			if(graphics.isWindowActive() && !graphics.isLost())
			{
				// rendern
				if(p_currentGS) p_currentGS->render(dt);

				// Ereignisse an den aktuellen Spielzustand weiterleiten
				if(p_currentGS)
				{
					for(vector<InputEvent>::const_iterator i = events.begin(); i != events.end(); i++)
					{
						p_currentGS->onInputEvent(*i);
					}
				}

				if(useFixedLogicStep)
				{
					while(logicTimeLeft >= fixedLogicStep)
					{
						if(!paused) Timer::updateAllTimers(fixedLogicStep);
						if(p_currentGS) p_currentGS->update(fixedLogicStep);
						logicTimeLeft -= fixedLogicStep;
					}
				}
				else
				{
					// alle Timer aktualisieren, wenn das Spiel nicht angehalten ist
					if(!paused) Timer::updateAllTimers(dt);

					// Spielzustand aktualisieren
					if(p_currentGS) p_currentGS->update(dt);

				}
			}

			if( !graphics.isWindowActive() )
			{
				Sleep( 100 );
			}

			LONGLONG endTime;
			QueryPerformanceCounter(reinterpret_cast<LARGE_INTEGER*>(&endTime));
			dt = endTime != startTime ? static_cast<float>(endTime - startTime) / counterFreq : 0.0f;

			if(useFixedLogicStep) logicTimeLeft += dt;
		}
	}

	void Game::quit()
	{
		PostQuitMessage(0);
	}

	void Game::pause()
	{
		if(paused) return;
		paused = true;
		DSoundResource::freeze();
	}

	void Game::resume()
	{
		if(!paused) return;
		paused = false;
		DSoundResource::unfreeze();
	}
}
