/*	                     __      __
	                     \ \    / /
	                      \ \  / /
	         _      _______\ \/ /_______      _
	        / \    / __________________ \    / \
	       (   \  | /        __        \ |  /   )
	     ___\  |  | |       / _\       | |  |  /___
	    |____  |  | |      ( <_        | |  | _____|
	      / \| |  | |       \__/       | |  | |/ \
	     / \ | |  | \__________________/ |  | | / \
	    /_/ \__/   \____________________/   \__/ \_\
	 ___  _   _ ___ __  __   _   _  _ ___   _     ____ __
	|   \| | | / __|  \/  | /_\ | \| |_ _| /_\   |__  /  \
	| |) | |_| \__ \ |\/| |/ _ \| .` || | / _ \    / / () |
	|___/ \___/|___/_|  |_/_/ \_\_|\_|___/_/ \_\  /_(_)__/

	      Framework fr den Spieleprogrammierwettbewerb
	             by David Scherfgen (TomasRiker)
*/

#ifndef _GAMESTATE_H
#define _GAMESTATE_H

#include <string>
#include "input.h"

namespace FrameWork
{
	// abstrakte Klasse fr einen Spielzustand
	class GameState
	{
		friend class Game;

	public:
		GameState();
		virtual ~GameState();

		// liefert den Namen des Spielzustands (z.B. "Hauptmen")
		virtual std::string getName() const = 0;

		// wird beim Betreten aufgerufen
		virtual void onEnter(void* p_context)
		{
		}

		// wird beim Verlassen aufgerufen (quit gibt an, ob die Anwendung beendet wird)
		virtual void onLeave(void* p_context,
							 bool quit)
		{
		}

		// wird aufgerufen, wenn ein Eingabeereignis aufgetreten ist
		virtual void onInputEvent(const InputEvent& evt)
		{
		}

		// Hier wird der Spielzustand gerendert.
		// dt enthlt die seit dem letzten Frame vergangene Zeit in Sekunden.
		virtual void render(float dt)
		{
		}

		// Hier wird der Spielzustand aktualisiert.
		// dt enthlt die seit dem letzten Frame vergangene Zeit in Sekunden.
		virtual void update(float dt)
		{
		}

		Game* getGame() {return p_game;}

	private:
		Game* p_game;
	};
}

#endif
