/*	                     __      __
	                     \ \    / /
	                      \ \  / /
	         _      _______\ \/ /_______      _
	        / \    / __________________ \    / \
	       (   \  | /        __        \ |  /   )
	     ___\  |  | |       / _\       | |  |  /___
	    |____  |  | |      ( <_        | |  | _____|
	      / \| |  | |       \__/       | |  | |/ \
	     / \ | |  | \__________________/ |  | | / \
	    /_/ \__/   \____________________/   \__/ \_\
	 ___  _   _ ___ __  __   _   _  _ ___   _     ____ __
	|   \| | | / __|  \/  | /_\ | \| |_ _| /_\   |__  /  \
	| |) | |_| \__ \ |\/| |/ _ \| .` || | / _ \    / / () |
	|___/ \___/|___/_|  |_/_/ \_\_|\_|___/_/ \_\  /_(_)__/

	      Framework fr den Spieleprogrammierwettbewerb
	             by David Scherfgen (TomasRiker)
*/

#ifndef _GRAPHICS_H
#define _GRAPHICS_H

#include <windows.h>
#include <d3d9.h>
#include <d3dx9.h>
#include <string>
#include <vector>
#include "singleton.h"

namespace FrameWork
{
	class Surface;

	// Grafik-Flags
	enum GraphicsFlags
	{
		GF_VSYNC = 1,		// vertikale Strahlsynchronisation einschalten
		GF_WINDOWED = 2,	// Fenstermodus anstatt Vollbildmodus
		GF_REFRAST = 4		// Reference-Rasterizer fr Darstellung verwenden
	};

	// Klasse fr Grafik und Fenster
	class Graphics : public Singleton<Graphics>
	{
		friend class Surface;
		friend LRESULT WINAPI windowProc(HWND window, unsigned int msg, WPARAM wParam, LPARAM lParam);

	public:
		Graphics(int width, int height, int bpp, DWORD flags, const std::string& windowTitle);
		~Graphics();

		// verarbeitet die Fensternachrichten
		bool processWindowMessages();

		// versucht, Direct3D und alle Texturen wiederherzustellen
		void tryToRestore();

		// stellt den Inhalt des Back-Buffers dar
		void showBackBuffer(float alpha = 1.0f);

		// speichert einen Screenshot (eine Nummer wird automatisch angehngt)
		void saveScreenshot(const std::string& filenameBegin, const std::string& fileFormat = "jpg");

		void setCameraPosition(const D3DXVECTOR2& cameraPosition) {this->cameraPosition = cameraPosition;}
		int getWidth() const {return width;}
		int getHeight() const {return height;}
		int getBPP() const {return bpp;}
		DWORD getFlags() const {return flags;}
		HWND getWindow() {return window;}
		bool isWindowActive() const {return active;}
		bool isLost() const {return lost;}
		IDirect3D9* getD3D() {return p_d3d;}
		IDirect3DDevice9* getD3DDevice() {return p_d3dDevice;}
		const D3DPRESENT_PARAMETERS& getD3DPP() const {return d3dpp;}
		Surface* getBackBuffer() {return p_backBuffer;}
		const D3DXVECTOR2& getCameraPosition() const {return cameraPosition;}

	private:
		void setDefaults();
		void clearBackBuffer();
		D3DFORMAT findBestBBFormat(D3DDEVTYPE devType, int bpp);
		unsigned int findBestRefreshRate(int width, int height, D3DFORMAT bbFormat, unsigned int maxRefreshRate);
		void setTexture(IDirect3DTexture9* p_texture);

		bool initialized;
		int width, height, bpp;
		DWORD flags;
		std::string windowTitle;
		HWND window;
		WNDCLASSEX windowClass;
		int windowWidth, windowHeight;
		bool active;
		bool lost;
		IDirect3D9* p_d3d;
		IDirect3DDevice9* p_d3dDevice;
		D3DPRESENT_PARAMETERS d3dpp;
		Surface* p_backBuffer;
		D3DXVECTOR2 cameraPosition;

		IDirect3DTexture9* p_currentTexture;
	};

	LRESULT WINAPI windowProc(HWND window, unsigned int msg, WPARAM wParam, LPARAM lParam);
}

#endif
