/*	                     __      __
	                     \ \    / /
	                      \ \  / /
	         _      _______\ \/ /_______      _
	        / \    / __________________ \    / \
	       (   \  | /        __        \ |  /   )
	     ___\  |  | |       / _\       | |  |  /___
	    |____  |  | |      ( <_        | |  | _____|
	      / \| |  | |       \__/       | |  | |/ \
	     / \ | |  | \__________________/ |  | | / \
	    /_/ \__/   \____________________/   \__/ \_\
	 ___  _   _ ___ __  __   _   _  _ ___   _     ____ __
	|   \| | | / __|  \/  | /_\ | \| |_ _| /_\   |__  /  \
	| |) | |_| \__ \ |\/| |/ _ \| .` || | / _ \    / / () |
	|___/ \___/|___/_|  |_/_/ \_\_|\_|___/_/ \_\  /_(_)__/

	      Framework fr den Spieleprogrammierwettbewerb
	             by David Scherfgen (TomasRiker)
*/

#ifndef _PARTICLEEMITTER_H
#define _PARTICLEEMITTER_H

#include "particlesystem.h"

namespace FrameWork
{
	// Emitter-Parameter
	struct ParticleEmitterParams
	{
		float lifetime;				// Lebensdauer der Partikel
		D3DXVECTOR2 offset;			// Offset der Partikel
		D3DXVECTOR2 direction;		// Richtung der Partikel
		float speed;				// Geschwindigkeit der Partikel
		D3DXCOLOR startColor;		// Startfarbe der Partikel
		D3DXCOLOR endColor;			// Endfarbe der Partikel
		float startOrientation;		// Startrotation der Partikel
		float endOrientation;		// Endrotation der Partikel
		float startSize;			// Startgre der Partikel
		float endSize;				// Endgre der Partikel
		float gravityInfluence;		// Gravitationseinfluss auf die Partikel
		float damping;				// Dmpfung der Partikelgeschwindigkeit
	};

	// Klasse fr einen Partikel-Emitter
	class ParticleEmitter
	{
		friend class ParticleSystem;

	public:
		void activate() {active = true;}
		void deactivate() {active = false;}
		bool isActive() const {return active;}
		bool toBeDestroyed() const {return autoDestroy < 0.0f && autoDestroy != -1.0f;}
		void setAutoDestroy(float autoDestroy) {this->autoDestroy = autoDestroy;}
		void setPosition(const D3DXVECTOR2& position) {this->position = position;}
		void setVelocity(const D3DXVECTOR2& velocity) {this->velocity = velocity;}
		void setParticleRate(float particleRate) {this->particleRate = particleRate;}
		void setBaseParams(const ParticleEmitterParams& base) {this->base = base;}
		void setDevianceParams(const ParticleEmitterParams& deviance) {this->deviance = deviance;}
		float getAutoDestroy() const {return autoDestroy;}
		const D3DXVECTOR2& getPosition() const {return position;}
		const D3DXVECTOR2& getVelocity() const {return velocity;}
		float getParticleRate() const {return particleRate;}
		const ParticleEmitterParams& getBaseParams() const {return base;}
		const ParticleEmitterParams& getDevianceParams() const {return deviance;}

		void setParticleLifetime(float lifetime, float deviance = 0.0f) {this->base.lifetime = lifetime; this->deviance.lifetime = deviance;}
		void setParticleDirection(const D3DXVECTOR2& direction, const D3DXVECTOR2& deviance = D3DXVECTOR2(0.0f, 0.0f)) {this->base.direction = direction; this->deviance.direction = deviance;}
		void setParticleOffset(const D3DXVECTOR2& offset, const D3DXVECTOR2& deviance = D3DXVECTOR2(0.0f, 0.0f)) {this->base.offset = offset; this->deviance.offset = deviance;}
		void setParticleSpeed(float speed, float deviance = 0.0f) {this->base.speed = speed; this->deviance.speed = deviance;}
		void setParticleStartColor(const D3DXCOLOR& startColor, const D3DXCOLOR& deviance = D3DXCOLOR(0.0f, 0.0f, 0.0f, 0.0f)) {this->base.startColor = startColor; this->deviance.startColor = deviance;}
		void setParticleEndColor(const D3DXCOLOR& endColor, const D3DXCOLOR& deviance = D3DXCOLOR(0.0f, 0.0f, 0.0f, 0.0f)) {this->base.endColor = endColor; this->deviance.endColor = deviance;}
		void setParticleStartOrientation(float startOrientation, float deviance = 0.0f) {this->base.startOrientation = startOrientation; this->deviance.startOrientation = deviance;}
		void setParticleEndOrientation(float endOrientation, float deviance = 0.0f) {this->base.endOrientation = endOrientation; this->deviance.endOrientation = deviance;}
		void setParticleStartSize(float startSize, float deviance = 0.0f) {this->base.startSize = startSize; this->deviance.startSize = deviance;}
		void setParticleEndSize(float endSize, float deviance = 0.0f) {this->base.endSize = endSize; this->deviance.endSize = deviance;}
		void setParticleGravityInfluence(float gravityInfluence, float deviance = 0.0f) {this->base.gravityInfluence = gravityInfluence; this->deviance.gravityInfluence = deviance;}
		void setParticleDamping(float damping, float deviance = 0.0f) {this->base.damping = damping; this->deviance.damping = deviance;}

	private:
		ParticleEmitter(const ivec2& spriteStartOnTex, const ivec2& spriteSizeOnTex);
		~ParticleEmitter();

		void emit(std::list<Particle>& output, float dt);

		bool active;
		float autoDestroy;
		ivec2 spriteStartOnTex;
		ivec2 spriteSizeOnTex;
		D3DXVECTOR2 position;
		D3DXVECTOR2 velocity;
		float particleRate;
		float particlesToEmit;
		ParticleEmitterParams base;
		ParticleEmitterParams deviance;
	};
}

#endif
