/*	                     __      __
	                     \ \    / /
	                      \ \  / /
	         _      _______\ \/ /_______      _
	        / \    / __________________ \    / \
	       (   \  | /        __        \ |  /   )
	     ___\  |  | |       / _\       | |  |  /___
	    |____  |  | |      ( <_        | |  | _____|
	      / \| |  | |       \__/       | |  | |/ \
	     / \ | |  | \__________________/ |  | | / \
	    /_/ \__/   \____________________/   \__/ \_\
	 ___  _   _ ___ __  __   _   _  _ ___   _     ____ __
	|   \| | | / __|  \/  | /_\ | \| |_ _| /_\   |__  /  \
	| |) | |_| \__ \ |\/| |/ _ \| .` || | / _ \    / / () |
	|___/ \___/|___/_|  |_/_/ \_\_|\_|___/_/ \_\  /_(_)__/

	      Framework fr den Spieleprogrammierwettbewerb
	             by David Scherfgen (TomasRiker)
*/

#ifndef _PARTICLESYSTEM_H
#define _PARTICLESYSTEM_H

#include <list>
#include <d3dx9.h>
#include "ivec2.h"

namespace FrameWork
{
	class Surface;
	class ParticleEmitter;

	// ein einzelner Partikel
	struct Particle
	{
		float lifetimeLeft;
		ivec2 spriteStartOnTex;
		ivec2 spriteSizeOnTex;
		D3DXVECTOR2 position, velocity;
		D3DXCOLOR color, deltaColor;
		float orientation, rotation;
		float size, deltaSize;
		float damping;
		float gravityInfluence;
	};

	// Klasse fr ein Partikelsystem
	class ParticleSystem
	{
	public:
		ParticleSystem(Surface* p_sprites);
		~ParticleSystem();

		// Erzeugt einen neuen Partikel-Emitter.
		// Die beiden Parameter bezeichnen die Position und die Gre des Partikel-Sprites auf
		// der beim Erzeugen des Partikelsystems angegebenen Textur.
		ParticleEmitter* createEmitter(const ivec2& spriteStartOnTex, const ivec2& spriteSizeOnTex);

		// lscht einen Partikel-Emitter
		void deleteEmitter(ParticleEmitter* p_emitter);

		// lscht alle Partikel-Emitter
		void deleteAllEmitters();

		// lscht alle Partikel
		void clear() {particles.clear();}

		// prft, ob der angegebene Emitter gltig ist
		bool isValidEmitter(const ParticleEmitter* p_emitter) const;

		// rendern
		void render(Surface* p_dest);

		// aktualisieren
		void update(float dt);

		void setGravity(const D3DXVECTOR2& gravity) {this->gravity = gravity;}
		const D3DXVECTOR2& getGravity() const {return gravity;}

	private:
		Surface* p_sprites;
		D3DXVECTOR2 gravity;
		std::list<ParticleEmitter*> emitters;
		std::list<Particle> particles;
	};
}

#endif
